@@ -11,7 +11,7 @@
 /*                                                                       */
 /*************************************************************************/
 
-/* stuff for transforming a dictionary entry into a disjunct list */
+/* Code that transforms a dictionary entry into a disjunct list */
 
 #include "build-disjuncts.h"
 #include "connectors.h"
@@ -124,25 +124,49 @@ static void debug_last(Clause *c, Clause **c_last, const char *type)
 #endif
 }
 
+#ifdef DEBUG
+/**
+ * Return the number of clauses that would be generated by expanding
+ * the expression.
+ */
+static unsigned long count_clauses(Exp *e)
+{
+	if (e->type == AND_type)
+	{
+		unsigned long cnt = 1;
+		for (Exp *opd = e->operand_first; opd != NULL; opd = opd->operand_next)
+			cnt *= count_clauses(opd);
+		return cnt;
+	}
+
+	if (e->type == OR_type)
+	{
+		unsigned long cnt = 0;
+		for (Exp *opd = e->operand_first; opd != NULL; opd = opd->operand_next)
+			cnt += count_clauses(opd);
+		return cnt;
+	}
+	return 1;
+}
+#endif
+
 /**
  * Build the clause for the expression e.  Does not change e
  */
 static Clause * build_clause(Exp *e, clause_context *ct, Clause **c_last)
 {
-	Clause *c = NULL;
-
 	assert(e != NULL, "build_clause called with null parameter");
 	if (e->type == AND_type)
 	{
-		Clause *c1 = pool_alloc(ct->Clause_pool);
-		c1->c = NULL;
-		c1->next = NULL;
-		c1->totcost = 0.0;
+		Clause *c = pool_alloc(ct->Clause_pool);
+		c->c = NULL;
+		c->next = NULL;
+		c->totcost = 0.0;
 		for (Exp *opd = e->operand_first; opd != NULL; opd = opd->operand_next)
 		{
 			Clause *c2 = build_clause(opd, ct, NULL);
 			Clause *c_head = NULL;
-			for (Clause *c3 = c1; c3 != NULL; c3 = c3->next)
+			for (Clause *c3 = c; c3 != NULL; c3 = c3->next)
 			{
 				for (Clause *c4 = c2; c4 != NULL; c4 = c4->next)
 				{
@@ -155,20 +179,24 @@ static Clause * build_clause(Exp *e, clause_context *ct, Clause **c_last)
 				}
 			}
 #if BUILD_DISJUNCTS_FREE_INETERMEDIATE_MEMORY /* Undefined - CPU overhead. */
-			free_clause_list(c1, ct);
+			free_clause_list(c, ct);
 			free_clause_list(c2, ct);
 #endif
-			c1 = c_head;
+			c = c_head;
 		}
-		c = c1;
 		if ((c != NULL) && (c->next == NULL) && (c_last != NULL)) *c_last = c;
 		debug_last(c, c_last, "AND_type");
+
+		for (Clause *c1 = c; c1 != NULL; c1 = c1->next)
+			c1->totcost += e->cost;
+		return c;
 	}
-	else if (e->type == OR_type)
+
+	if (e->type == OR_type)
 	{
 		Clause *or_last = NULL;
+		Clause *c = build_clause(e->operand_first, ct, &or_last);
 
-		c = build_clause(e->operand_first, ct, &or_last);
 		/* We'll concatenate the lists of clauses. */
 		for (Exp *opd = e->operand_first->operand_next; opd != NULL; opd = opd->operand_next)
 		{
@@ -179,33 +207,31 @@ static Clause * build_clause(Exp *e, clause_context *ct, Clause **c_last)
 
 		if (c_last != NULL) *c_last = or_last;
 		debug_last(c, c_last, "OR_type");
+
+		for (Clause *c1 = c; c1 != NULL; c1 = c1->next)
+			c1->totcost += e->cost;
+		return c;
 	}
-	else if (e->type == CONNECTOR_type)
+
+	if (e->type == CONNECTOR_type)
 	{
-		c = pool_alloc(ct->Clause_pool);
+		Clause *c = pool_alloc(ct->Clause_pool);
 		c->c = build_terminal(e, ct);
-		c->totcost = 0.0;
 		c->next = NULL;
+		c->totcost = e->cost;
 		if (c_last != NULL) *c_last = c;
-	}
-	else
-	{
-		assert(false, "Unknown expression type %d", (int)e->type);
+		return c;
 	}
 
-	/* c now points to the list of clauses */
-	for (Clause *c1 = c; c1 != NULL; c1 = c1->next)
-		c1->totcost += e->cost;
-
-	return c;
+	assert(false, "Unknown expression type %d", (int)e->type);
 }
 
 /**
- * Build a disjunct list out of the clause list c.
- * string is the print name of word that generated this disjunct.
+ * Build a disjunct list out of the clause list cl.
+ * wstring is the print name of word that generated this disjunct.
  */
 static Disjunct *
-build_disjunct(Sentence sent, Clause * cl, const char * string,
+build_disjunct(Sentence sent, Clause * cl, const char * wstring,
                const gword_set *gs, float cost_cutoff, Parse_Options opts)
 {
 	Pool_desc *connector_pool = sent->Connector_pool;
@@ -219,11 +245,10 @@ build_disjunct(Sentence sent, Clause * cl, const char * string,
 	Disjunct *dis = NULL;
 	for (; cl != NULL; cl = cl->next)
 	{
-		Disjunct *ndis;
-
 		if (unlikely(NULL == cl->c)) continue; /* no connectors */
 		if (cl->totcost > cost_cutoff) continue;
 
+		Disjunct *ndis;
 #if USE_SAT_SOLVER
 		if (sat_solver) /* For the SAT-parser, until fixed. */
 			ndis = xalloc(sizeof(Disjunct));
@@ -251,7 +276,7 @@ build_disjunct(Sentence sent, Clause * cl, const char * string,
 				continue;
 			}
 
-			Connector *n = connector_new(connector_pool, t->e->condesc, opts);
+			Connector *n = connector_new(connector_pool, t->e->condesc);
 			t->tracon = n; /* cache this tracon */
 
 			n->exp_pos = t->e->pos;
@@ -264,9 +289,9 @@ build_disjunct(Sentence sent, Clause * cl, const char * string,
 
 		/* XXX add_category() starts category strings by ' '.
 		 * FIXME Replace it by a better indication. */
-		if (sat_solver || (!IS_GENERATION(sent->dict) || (' ' != string[0])))
+		if (sat_solver || (!IS_GENERATION(sent->dict) || (' ' != wstring[0])))
 		{
-			ndis->word_string = string;
+			ndis->word_string = wstring;
 			ndis->cost = cl->totcost;
 			ndis->is_category = 0;
 		}
@@ -276,7 +301,7 @@ build_disjunct(Sentence sent, Clause * cl, const char * string,
 			ndis->category =
 				malloc(sizeof(*ndis->category) * ndis->num_categories_alloced);
 			ndis->num_categories = 1;
-			ndis->category[0].num = strtol(string, NULL, 16);
+			ndis->category[0].num = strtol(wstring, NULL, 16);
 			ndis->category[1].num = 0; /* API array terminator */
 			assert(sat_solver || ((ndis->category[0].num > 0) &&
 			       (ndis->category[0].num < 64*1024)),
@@ -324,7 +349,37 @@ Disjunct *build_disjuncts_for_exp(Sentence sent, Exp* exp, const char *word,
 	// print_disjunct_list(dis);
 	pool_reuse(ct.Clause_pool);
 	pool_reuse(ct.Tconnector_pool);
-	return dis;
+
+	/* We are done, in the concvetional case. */
+	if (NULL == opts || 0 == opts->max_disjuncts) return dis;
+
+	/* If there are more than the allowed number of disjuncts,
+	 * then randomly discard some of them. The discard is done
+	 * with uniform weighting; no attempt to look at the cost
+	 * is made. A fancier algo might selectively choose those
+	 * with lower cost.
+	 */
+	unsigned int maxdj = opts->max_disjuncts;
+	unsigned int discnt = count_disjuncts(dis);
+	if (discnt < maxdj) return dis;
+
+	/* If we are here, we need to trim down the list */
+	unsigned int rst = sent->rand_state;
+	Disjunct *kdis = dis;
+	Disjunct *ktail = dis;
+	for (Disjunct *d = dis->next; d != NULL; d=d->next)
+	{
+		unsigned int pick = rand_r(&rst) % discnt;
+		if (pick < maxdj)
+		{
+			ktail->next = d;
+			ktail = d;
+		}
+	}
+	ktail->next = NULL;
+	if (0 != sent->rand_state) sent->rand_state = rst;
+
+	return kdis;
 }
 
 #ifdef DEBUG
