@@ -1824,8 +1824,8 @@ bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
 
     // Allocate memory for the connectors, because they should persist
     // beyond the lifetime of the sat-solver data structures.
-    clink.lc = connector_new(NULL, NULL, NULL);
-    clink.rc = connector_new(NULL, NULL, NULL);
+    clink.lc = connector_new(NULL, NULL);
+    clink.rc = connector_new(NULL, NULL);
 
     *clink.lc = lpc->connector;
     *clink.rc = rpc->connector;
